#if !defined __event_h
#define __event_h

//#include <iostream.h>
//#include <windows.h>
//#include <stdio.h>


///////////////////////////////////////////////////////////
///   CODE MESSAGES
///////////////////////////////////////////////////////////
#define USER_NEW 1
#define USER_DEL 2
#define USER_ONLINE 3
#define USER_OFFLINE 4
#define DOC_NEW 5
#define DOC_DEL 6
#define DOC_START 7
#define DOC_STOP 8
#define BAD_COMMAND 70
#define GOOD_COMMAND 71
#define UPDATE_PLAYLIST_EVENT 13

///////////////////////////////////////////////////////////
///   Event
///////////////////////////////////////////////////////////
class Event
{
	//constructors
public:	
	Event(){ params = NULL; };
	Event(int msg, LPCTSTR * args, int nbargs);
	~Event();

    // Attributes
protected:
    int message; // message number
    LPCTSTR * params;
    int nbparams;
    
    //Operators
public:
    virtual LPCTSTR getMessage();// retourne le message correspondant au code message
	virtual int getMessageCode();// retourne le code message
    virtual LPCTSTR * getParams(); //retourne les paramtres
    virtual LPCTSTR getParam(int pos);
    virtual int getNbParams(); //retourne le nombre de paramtre

};

///////////////////////////////////////////////////////////
///   LISTENERS
///////////////////////////////////////////////////////////
class ServerListener {
public:
	virtual void OnNewEvent(Event* e)=0;
};


///////////////////////////////////////////////////////////
///   New USer Event
///////////////////////////////////////////////////////////

class NewUserEvent : public Event
{
	//constructors
public:
	
	NewUserEvent(LPCTSTR login){
		message = USER_NEW;
		
		params=new LPCTSTR[1];
		params[0] = login;
		nbparams = 1;
	}

};

///////////////////////////////////////////////////////////
///   DEl User Event
///////////////////////////////////////////////////////////
class DelUserEvent : public Event 
{
public:
	DelUserEvent(LPCTSTR login){
		message = USER_DEL;
		
		params = new LPCTSTR[1];
		params[0] = login;
		nbparams = 1;		
	}
};

///////////////////////////////////////////////////////////
///   Online User Event
///////////////////////////////////////////////////////////
class UserOnlineEvent : public Event 
{
public:
	UserOnlineEvent(LPCTSTR login){

		message = USER_ONLINE;
						
		params = new LPCTSTR[1];
		params[0] = login;
		nbparams = 1;
	}
};

///////////////////////////////////////////////////////////
///   Offline User Event
///////////////////////////////////////////////////////////
class UserOfflineEvent : public Event 
{
public:
	UserOfflineEvent(LPCTSTR login){

		message = USER_OFFLINE;
						
		params = new LPCTSTR[1];
		params[0] = login;
		nbparams = 1;
	}
};

///////////////////////////////////////////////////////////
///   New Doc Event
///////////////////////////////////////////////////////////
class NewDocEvent : public Event 
{
public:
	NewDocEvent(LPCTSTR owner, LPCTSTR filename){

		message = DOC_NEW;

		params = new LPCTSTR[2];
		params[0] = owner;
		params[1] = filename;		
		nbparams = 2;
	}
};

///////////////////////////////////////////////////////////
///   Start Sending Doc Event
///////////////////////////////////////////////////////////
class StartSendingDocEvent : public Event 
{
public:
	StartSendingDocEvent(LPCTSTR owner, LPCTSTR filename, LPCTSTR ip, int port){

		message = DOC_START;

		char * strPort = new char[10];
		sprintf(strPort,"%d",port);

		params = new LPCTSTR[4];
		params[0] = owner;
		params[1] = filename;
		params[2] = ip;
		params[3] = (LPCTSTR) strPort;
		nbparams = 4;
	}
};

///////////////////////////////////////////////////////////
///   Stop sending Doc Event
///////////////////////////////////////////////////////////
class StopSendingDocEvent : public Event 
{
public:
	StopSendingDocEvent(LPCTSTR owner, LPCTSTR filename){

		message = DOC_STOP;
		
		params = new LPCTSTR[2];
		params[0] = owner;
		params[1] = filename;
		nbparams = 2;
	}
};

///////////////////////////////////////////////////////////
///   Del Doc Event
///////////////////////////////////////////////////////////
class DelDocEvent : public Event 
{
public:
	DelDocEvent(LPCTSTR key){

		message = DOC_DEL;
		
		params = new LPCTSTR[1];
		params[0] = key;
		nbparams = 1;
	}
};

class BadCommandEvent : public Event 
{
public:
	BadCommandEvent(LPCTSTR errormsg){

		message = BAD_COMMAND;
		
		params = new LPCTSTR[1];
		params[0] = errormsg;
		nbparams = 1;
	}
};

class GoodCommandEvent : public Event 
{
public:
	GoodCommandEvent(LPCTSTR msg){

		message = GOOD_COMMAND;
		
		params = new LPCTSTR[1];
		params[0] = msg;
		nbparams = 1;
	}
};


class UpdatePlayListEvent : public Event
{
	//constructors
public:
	
	UpdatePlayListEvent(){
		message = UPDATE_PLAYLIST_EVENT;		
		nbparams = 0;
	}

};


#endif