// -----------------------------------------------------------------------
// A kind of Data table
// Data are stored in a 
// -----------------------------------------------------------------------
#ifndef _IRadDataTable_H_
#define _IRadDataTable_H_
// -----------------------------------------------------------------------

#include <afx.h>
#include <afxtempl.h>

// -----------------------------------------------------------------------
template <class T>
class IRadDataTable
{
  public:

  //-----------------------------------------------------------------------
  // IRadDataTable(CString FileName)
  // 
  // Create a data table, if the specified file exist, record
  // are added to the table, else, data table is empty.
  // 
  // @param CString : FileName
  //-----------------------------------------------------------------------
	IRadDataTable(CString FileName) : FileName(FileName)
	{
    TRY {    
      // Try to read a database file
      CFile dataFile(FileName, CFile::modeRead);

      // Create an archive to deserialize record
      CArchive archive(&dataFile, CArchive::load);

      // TODO : See if there is some error checking to do on CArchive

      // Retrieve number of record
      WORD nCount;
      archive >> nCount;

      // Retrieve each record
      while (nCount-- > 0)
      {
        T* record = new T;
        archive >> record;
        data.AddTail(record);
      }
      archive.Close();
      // Close file
      dataFile.Close();
    } CATCH ( CFileException, e ) {
      // Can't open file, so database is empty
    }
    END_CATCH
	}

	//-----------------------------------------------------------------------
  // ~IRadDataTable()
	//-----------------------------------------------------------------------
	~IRadDataTable() {
    Flush();
    ClearAll();
	}

	//-----------------------------------------------------------------------
  // bool AddRecord(T* rs)
  // 
  // @param T* : rs a recordset (must be serializable)
  // @return bool : added successfully
  //-----------------------------------------------------------------------
	bool AddRecord(T* rs) {
    bool result = false;

    if( FindRecord(rs->getKey()) == NULL)
    {
      data.AddTail(rs);
		  result = true;
    }

    return result;
	}

	//-----------------------------------------------------------------------
  // bool DeleteRecord(CString dkey)
  // 
  // @param CString : dkey key of the record to delete
  // @return bool : deleted successfully
  //-----------------------------------------------------------------------
	bool DeleteRecord(CString dkey) {
		T* result = NULL;
  
    POSITION pos_d = data.GetHeadPosition();
    // Keep the old position, because GetNext
    // modify the current position
    POSITION old_pos_d;

	  while (pos_d != NULL)
	  {
      old_pos_d = pos_d;
      result = (T*)data.GetNext(pos_d);
      if( result->getKey() == dkey ) 
      {
        delete result;
        data.RemoveAt(old_pos_d);
        return true;
      }
	  }

		return false;
	}

	//-----------------------------------------------------------------------
  // T* FindRecord(CString dkey)
  // 
  // @param CString : fkey key of the record to find
  // @return T* : the record or NULL if it was not found
  //-----------------------------------------------------------------------
	T* FindRecord(CString fkey) {
		T* result = NULL;
  
    POSITION pos_f = data.GetHeadPosition();
	  while (pos_f != NULL)
	  {
        result = (T*)data.GetNext(pos_f);
        if( result->getKey() == fkey )
        {
          // We found the record
          return result;
        }
	  }
    // The record was not found
		return NULL;
	}

	// -----------------------------------------------------------------------
	// void GotoFirstRecord()
  // 
  // Used at the beginning of an iteration, set the position to the first
  // record.
  //
  // @see : GetRecord()
	// -----------------------------------------------------------------------
	void GotoFirstRecord() {
    internal_pos = data.GetHeadPosition();
	}

	//-----------------------------------------------------------------------
  // T* GetRecord()
  //
  // Get the record on current position then increment position
  //
  // @return T* : the current record or NULL if we are at end of table
  //-----------------------------------------------------------------------
	T* GetRecord() {

		T* result = NULL;

    if( internal_pos != NULL ) 
    {
        result = (T*)data.GetNext(internal_pos);
    }

		return result;
	}

	// -----------------------------------------------------------------------
  // void Flush()
	//
  // Write the content of the table in the file, overwriting all previous
  // data
	// -----------------------------------------------------------------------
	void Flush() {
		// Write to the file

		// Create or overwrite file
		CFile dataFile(FileName, CFile::modeCreate | CFile::modeWrite);
		
		// Create an archive
		CArchive archive(&dataFile, CArchive::store);
		
		// Write number of record
		WORD nCount = (WORD)data.GetCount();
		archive << nCount;

		if(nCount > 0) {		
			// Start at the beginning
			POSITION pos_f = data.GetHeadPosition();
		
			// Write each record (they must me serializable, so extend CObject)
			while (pos_f != NULL)
			{
				archive << (T*)data.GetNext(pos_f);
				nCount--;
			}
			// Check if we have written the right number
			ASSERT(nCount == 0);
		}
		
		archive.Close();
		// Close the file
		dataFile.Close();
	}

	// -----------------------------------------------------------------------
  // void ClearAll()
	// -----------------------------------------------------------------------
	void ClearAll() {
    T* record;
    // Delete each record of the list
    while ( !data.IsEmpty() ) 
    {
      record = (T*)data.GetHead();
      data.RemoveHead();
      delete record;
    }
  }

	// -----------------------------------------------------------------------
  protected:
    CString FileName;
    CTypedPtrList<CPtrList, T*> data;
    POSITION internal_pos;
};

// -----------------------------------------------------------------------
#endif
// -----------------------------------------------------------------------