// ManageDocsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ServerAudio.h"
#include "ManageDocsDlg.h"
#include "AddFileDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NUM_COL 7
#define OWNER_COL 0
#define FILE_COL 2
#define DESCR_COL 3
#define STREAM_COL 5
#define PERSIST_COL 6

static LPTSTR _colLabel[NUM_COL] = {"Owner","Relative Path","File","Description","Mime Type","Streamable","Persisting"};

static int _colWidth[NUM_COL] = {100,150,100,265,70,40,40};

#define UNCHECK 0
#define CHECK 1
#define _YES "Yes"
#define _NO "No"


/////////////////////////////////////////////////////////////////////////////
// CManageDocsDlg dialog


CManageDocsDlg::CManageDocsDlg(CWnd* pParent /*=NULL*/, CServerAudioDoc * doc,int dlgType)
	: CDialog(CManageDocsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CManageDocsDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_document = doc;
	dialogType = dlgType;
}


void CManageDocsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CManageDocsDlg)
	DDX_Control(pDX, IDOK, m_Btn_Ok);
	DDX_Control(pDX, IDC_BTN_ADD, m_Btn_Add);
	DDX_Control(pDX, IDC_DOCUMENT_DELETE, m_Btn_Del);
	DDX_Control(pDX, IDC_CHK_STREAM, m_Stream);
	DDX_Control(pDX, IDC_CHK_PERSIST, m_Persist);
	DDX_Control(pDX, IDC_EDIT_DESCRIPTION, m_Edit_Description);
	DDX_Control(pDX, IDC_LIST, m_list);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CManageDocsDlg, CDialog)
	//{{AFX_MSG_MAP(CManageDocsDlg)
	ON_BN_CLICKED(IDC_DOCUMENT_DELETE, OnDocumentDelete)
	ON_BN_CLICKED(IDC_DOCUMENT_RENAME, OnDocumentRename)
	ON_NOTIFY(LVN_KEYDOWN, IDC_LIST, OnKeydownList)
	ON_NOTIFY(NM_CLICK, IDC_LIST, OnClickList)
	ON_BN_CLICKED(IDC_BTN_ADD, OnBtnAdd)
	ON_BN_CLICKED(IDC_BUTTON_CANCEL, OnButtonCancel)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CManageDocsDlg message handlers

BOOL CManageDocsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

  // Show full row selection
  DWORD dwStyle;
  dwStyle = m_list.SendMessage(LVM_GETEXTENDEDLISTVIEWSTYLE, 0 ,0);
  dwStyle |= LVS_EX_FULLROWSELECT /* |LVS_EX_GRIDLINES */;
  m_list.SendMessage( LVM_SETEXTENDEDLISTVIEWSTYLE, 0,dwStyle ); 

	// TODO: Add extra initialization here
	int i;
	LV_COLUMN lvc;
	for (i = 0;   i < NUM_COL;   i++) {
		lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT; //| LVCF_SUBITEM;

//	lvc.iSubItem = i;
		lvc.pszText = _colLabel[i];
		lvc.cx = _colWidth[i];
		lvc.fmt = LVCFMT_LEFT;

		m_list.InsertColumn(i, &lvc);
	}

	// refresh data list
	m_list.DeleteAllItems();
	Fill();

	// activate or desctivate buttons 
  if (dialogType == DIALOG_GETFILENAME) {
		m_Btn_Del.EnableWindow(FALSE);
    m_Btn_Ok.ShowWindow(TRUE);
  }
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CManageDocsDlg::Fill()
{

	IRadDataTable<CDocInfo> * lst = m_document->dbDoc->GetListAll();
	CDocInfo* current;
	int i=0;
	lst->GotoFirstRecord();

	// Clear l'affichage
	m_list.DeleteAllItems();

	while ((current = lst->GetRecord ()) != NULL) {
		
		// Ajoute un nouveau Item
		m_list.InsertItem(
		    LVIF_TEXT|LVIF_STATE, i, "newItem", 
			(i%2)==0 ? LVIS_SELECTED : 0, LVIS_SELECTED,
			0, 0);
		// modifie l'item
		m_list.SetItemText(i, 0, current->owner);
		m_list.SetItemText(i, 1, current->relativePath);
		m_list.SetItemText(i, 2, current->filename);
		m_list.SetItemText(i, 3, current->description);
		m_list.SetItemText(i, 4, current->mimetype);
		if (current->streamable == TRUE) 
			m_list.SetItemText(i, 5, _YES);
		else 
			m_list.SetItemText(i, 5, _NO);
		
		if (current->persistent == TRUE) 
			m_list.SetItemText(i, 6, _YES);
		else 
			m_list.SetItemText(i, 6, _NO);		
		i++;
	}	


}

void CManageDocsDlg::OnDocumentDelete() 
{
	// TODO: Add your control notification handler code here
	int sel = m_list.GetSelectionMark();
	if (sel >= 0) {		
		CString owner = m_list.GetItemText(sel,OWNER_COL);
		CString filename = m_list.GetItemText(sel,FILE_COL);

		m_list.DeleteItem(sel);
		m_list.SetSelectionMark (-1);

		m_document->dbDoc->UnRegisterDocument(owner+filename);

		m_document->dbDoc->Flush();
		//m_list.SetFocus ();
		//m_list.SetSelectionMark (pos);
		//m_list.SetItemState (pos, LVIS_SELECTED,LVIS_SELECTED);
	}

}

void CManageDocsDlg::OnDocumentRename() 
{
	
	int sel = m_list.GetSelectionMark();
	
	// create key
	CString owner, filename;
	owner = m_list.GetItemText(sel, OWNER_COL);
	filename = m_list.GetItemText(sel, FILE_COL);	

	CString temp;
	m_Edit_Description.GetWindowText(temp);
	m_list.SetItemText(sel, DESCR_COL, temp);

	CDocInfo* di = m_document->dbDoc->GetDocInfo(owner+filename);
	di->description = temp;
	m_document->dbDoc->Flush();


	if (m_Stream.GetCheck() == CHECK) {
		m_list.SetItemText(sel, STREAM_COL, _YES);

	} else {
		m_list.SetItemText(sel, STREAM_COL, _NO);
	}

	if (m_Persist.GetCheck() == CHECK) {
		m_list.SetItemText(sel, PERSIST_COL, _YES);
		
	} else {
		m_list.SetItemText(sel, PERSIST_COL, _NO);
	}

}

void CManageDocsDlg::OnKeydownList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_KEYDOWN* pLVKeyDow = (LV_KEYDOWN*)pNMHDR;
	// TODO: Add your control notification handler code here
		int sel = m_list.GetSelectionMark();
	
	if (sel >= 0) {
		m_Edit_Description.SetWindowText(m_list.GetItemText(sel, DESCR_COL));
		if (m_list.GetItemText(sel, STREAM_COL) == _YES)
			m_Stream.SetCheck(CHECK);
		else 
			m_Stream.SetCheck(UNCHECK);

		if (m_list.GetItemText(sel, PERSIST_COL) == _YES)
			m_Persist.SetCheck(CHECK);
		else 
			m_Persist.SetCheck(UNCHECK);
	}
	*pResult = 0;
}



void CManageDocsDlg::OnClickList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	int sel = m_list.GetSelectionMark();
	
	if (sel >= 0) {
		m_Edit_Description.SetWindowText(m_list.GetItemText(sel, DESCR_COL));
		if (m_list.GetItemText(sel, STREAM_COL) == _YES)
			m_Stream.SetCheck(CHECK);
		else 
			m_Stream.SetCheck(UNCHECK);

		if (m_list.GetItemText(sel, PERSIST_COL) == _YES)
			m_Persist.SetCheck(CHECK);
		else 
			m_Persist.SetCheck(UNCHECK);
	}
	*pResult = 0;
}

void CManageDocsDlg::OnOK() 
{
	// TODO: Add extra validation here
	if (dialogType == DIALOG_GETFILENAME)
		if (m_list.GetSelectionMark() < 0) {
      MessageBox("You have to select a file in the file list.","PlayList add file",MB_ICONEXCLAMATION|MB_OK);
      return;
		} else {
			int sel = m_list.GetSelectionMark();
			CString filename,owner;
			owner = m_list.GetItemText(sel, OWNER_COL);
			filename = m_list.GetItemText(sel, FILE_COL);
			// TO DO :get into database	
			CDocInfo* temp = m_document->dbDoc->GetDocInfo(owner+filename);
			if (temp == NULL) {
				MessageBox("Unknow file "+filename,"Database Error",MB_OK);
				CDialog::OnCancel();
			} else  {
				selectedDoc  = *temp;
			}
			//CDocInfo temp("monFichier.zip", "Toff", "\\therep\\","zip",FALSE,FALSE,"Le fichier de Toff a lui");
		//	selectedDoc = temp;

		}


		
	CDialog::OnOK();
}

void CManageDocsDlg::OnBtnAdd() 
{
	// TODO: Add your control notification handler code here
	// load dialog box
//	SetCurrentDirectory(m_document->GetDocumentPath());
	CAddFileDlg cafd(NULL);
  cafd.SetInitialDir(m_document->GetDocumentPath());
	
	if (cafd.DoModal () == IDOK) {
		//add the file to the database
		//MessageBox("fichier choisi","ok!",MB_OK);
		//CFile f(cfd.GetPathName (), CFile::modeRead);
		//cafd.UpdateData(TRUE);
//		CString tmp= CString(cafd.GetFilename()+ cafd.GetOwner()+ cafd.GetRelativePath()+ cafd.GetMimeType()+  cafd.GetDescription());
		//MessageBox(tmp.GetBuffer(0),"",MB_OK);
		CDocInfo * doci= new CDocInfo(cafd.GetFilename(), cafd.GetOwner(), cafd.GetRelativePath(), cafd.GetMimeType(), cafd.GetStreamable(), cafd.GetPersistent(), cafd.GetDescription());
		if(m_document->dbDoc->RegisterDocument(doci)==FALSE)
			MessageBox("pb : peut-te dj insr dns la database","pb",MB_OK);
		
		m_document->dbDoc->Flush();
		Fill();

		//m_document_db_file_o.SetWindowText(cfd.GetPathName ());
		//GetDocument ()->SetDocumentDbFile(cfd.GetPathName ());
	}

}

void CManageDocsDlg::OnButtonCancel() 
{
	// TODO: Add your control notification handler code here
	CDialog::OnCancel();  
}
