// PlayList.cpp: implementation of the CPlayList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PlayList.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPlayList::CPlayList(): mutex(FALSE, "PlayListMutex", NULL){
};


CPlayList::~CPlayList() {
};

//
CDocInfo CPlayList::GetHead(){
	mutex.Lock ();
	CDocInfo temp = playlst.GetHead ();
	mutex.Unlock ();
	return temp;
};

//
void CPlayList::Enqueue(CDocInfo doc){
	mutex.Lock ();
	playlst.AddTail(doc);
	mutex.Unlock ();
};

//
int CPlayList::ChangePriority(int elem, int incr){
	mutex.Lock ();
	int result = 0;
	CDocInfo temp = playlst.GetAt(playlst.FindIndex(elem));
	playlst.RemoveAt(playlst.FindIndex(elem));

	if (elem - incr < 1) {
		playlst.AddHead(temp);
	} else if (elem - incr > playlst.GetCount ()) {
		playlst.AddTail(temp);
		result = playlst.GetCount () - 1;
	} else {
		playlst.InsertAfter(playlst.FindIndex(elem-incr-1), temp);
		result = elem-incr;
	}
	mutex.Unlock ();
	return result;
};

//
CDocInfo CPlayList::GetElementAt(int pos){
	mutex.Lock ();
	CDocInfo temp = playlst.GetAt(playlst.FindIndex(pos));
	mutex.Unlock ();
	return temp;
};

//
BOOL CPlayList::RemoveElementAt(int pos){
	if ( pos < 0 || pos > playlst.GetCount ()) return false;
	mutex.Lock ();
	playlst.RemoveAt(playlst.FindIndex(pos));
	mutex.Unlock ();
	return true;
};

// remove all elements
void CPlayList::RemoveAll(){
	mutex.Lock ();
	playlst.RemoveAll();
	mutex.Unlock ();
};

// get playlist size
int CPlayList::Size(){
	return playlst.GetCount ();
};

// remove an element corresponding to the key
BOOL CPlayList::RemoveElement(CString key) {
	mutex.Lock ();
	BOOL result = FALSE;
	CDocInfo temp;
	POSITION index;

	for (int i=0 ;   i < playlst.GetCount() && !result;   i++) {
		index = playlst.FindIndex(i);
		temp  = playlst.GetAt(index);
		if (temp.getKey() == key) {
			playlst.RemoveAt(index);
			result = TRUE;
		}
	}
	mutex.Unlock ();
	return result;
}
