#include "stdafx.h" 
#include "ProtocolServer.h"
#include "Server.h"


//////////////////////////////////////////////////////////////////////////////////////
//@info																				//
//		TCP mode:	The Server receives a command , parse it, and					//
//					do the appropriate operations (via the invoke method).			//
//					The result of the invoke method is send back to the				//
//					client.															//
//																					//
//					The possible commands received by the server are: 				//
//					LOGIN <login> <password> <ip> <port>							//
//					LOGOUT <login>													//
//					WHOIS <login>													//
//					REGISTER <login> <pass> <name> <lastname> <ip> <port>			//
//																					//
//																					//
//		MULTICAST mode:	The Server sends some commands to the clients.				//
//						The Server creates the request via the createCommand method	//
//						and send it to the clients.									//
//////////////////////////////////////////////////////////////////////////////////////
  

/////////////////////////////////////////////////////////////////////////////////////////////
//@info	These are the different return code sent by the server to the client.
//		BASE_<??> + CODE_<command> 
//		
//		if ?? is OK then the operation succeed else if ?? is KO then the 
//		operation failed.
//		The <command> is the command received from the client

#define BASE_OK 200
#define BASE_KO 400


#define CODE_LOGIN 1
#define CODE_LOGOUT 2
#define CODE_WHOIS 3
#define CODE_REGISTER 4
#define CODE_BADPARAMETERS 5
#define CODE_UNKNOWNCOMMAND 6
#define CODE_DOCINFO 9
#define CODE_PLAYLIST 10

/////////////////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////////////////////////////////
////																									////
////	CLASS PARSER																					////
////	IRADIS 2001		 																				////
////																									////
////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Constructor 
//@args char mytoken The token used by the parser
Parser::Parser(char mytoken):token(mytoken) { }
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//extractParams()
//@info Parse the CString 
//@args CString *cs	the CString to parse
//@return CStringList*	A List of arguments. The first element of the list is the command. 
CStringList* Parser::extractParams(CString *cs){
	CStringList *params = new CStringList();
	int start=0;
	int oldstart=0;
	int index=0;

	while(index >= 0){
		index=cs->Find(token ,start);
		if (index >= 0) params->AddTail(cs->Mid(start,index-start));
		oldstart=start;
		start = index + 1;
	}

	if(cs->Mid(oldstart).GetLength() > 0 || params->GetCount() == 0)
		params->AddTail(cs->Mid(oldstart));

	OutputDebugString("Protocol::extractParams\n");
	return params; 
}
/////////////////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////////////////////////////////
////																									////
////	CLASS IRADIS PROTOCOL SERVER																	////
////	IRADIS 2001		 																				////
////																									////
////////////////////////////////////////////////////////////////////////////////////////////////////////////

//@info This is our communication protocol. If you want to change it you just have
//to define an other class that extends ProtocolServer.
//

/////////////////////////////////////////////////////////////////////////////////////////////
//Constructor
//@info The protocole used in our application
//@args	ServerRunner *sr We need an instance of the server in order to access to the databases and
//		do correctly the invokations.
//@args	Parser *pr The parser that will allows us to parse the command into arguments
IradisProtocolServer::IradisProtocolServer(ServerRunner *sr, Parser *pr) : server(sr), parser(pr) { 

}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//CreateCommand()
//@info Allow us to create a command separated by tokens
//		Used when we want to send a MULTICAST command
//@args CStringList *params The arguments of the final command
//@return CString*	The command 
CString* IradisProtocolServer::createCommand(CStringList* params){
	CString res;
/*	
    POSITION pos_d = params->GetHeadPosition();

	while (pos_d != NULL)
	{
	  res += *(params->GetNext(pos_d));
	  res += CString(parser->token);
	}*/

	for(int i=0; i<params->GetCount();i++) { 
		if (i+1 == params->GetCount())
			res = res + params->GetAt(params->FindIndex(i));
		else
			res = res + params->GetAt(params->FindIndex(i)) + CString(parser->token);
	}

	return new CString(res);
}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//invokeLogin()
//@info Used by invoke (@see invoke). Allow to do the appropriate operations
//		when the command sent is LOGIN.
//@args CStringList *params The list which defines the command
//@return CString*	The return code which shows if the command has or not suceeded
CString* IradisProtocolServer::invokeLOGIN(CStringList* params, CTCPDialogThread* tcpThread) {
     cout << "Protocol::invokeLOGIN" << endl;
	CString RemoteLogin=params->GetAt(params->FindIndex(1));
	CString RemotePass=params->GetAt(params->FindIndex(2));
	CString RemoteIP=params->GetAt(params->FindIndex(3));
	CString RemotePort=params->GetAt(params->FindIndex(4));

	tcpThread->login = RemoteLogin;

	CUserInfo* user = server->userDB->GetUserInfo(RemoteLogin);
 
	if (  user == NULL){
		//The User isn't registered, he must register in order to connect
		//return new CString(BASE_KO + CODE_LOGIN + "");
			
			CString * res=new CString();
			res->Format(_T("%d"),BASE_KO + CODE_LOGIN);
			cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
			CString tmp="LOGIN: An unregistered user tried to connect under the login :"+RemoteLogin;
			server->Notify(new BadCommandEvent(tmp.GetBuffer(0)));
			return res;
	}else{
		//We must check the user (user != null)
		if ( RemotePass == user->passwd && !user->banned && !user->connected ){
				user->ip = RemoteIP;
				user->connected = true;
				user->port = atoi(RemotePort.GetBuffer(0));
				CString * res=new CString();
				res->Format(_T("%d"),BASE_OK + CODE_LOGIN);
				cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
				server->Notify(new UserOnlineEvent(user->login.GetBuffer(0)));
				server->SendPlaylist();
				return res;

		} else {
			CString * res=new CString();
			res->Format(_T("%d"),BASE_KO + CODE_LOGIN);
			cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
			CString tmp="LOGIN: An user tried to connect under the login "+RemoteLogin+" but he wasn't allowed to.";
			server->Notify(new BadCommandEvent(tmp.GetBuffer(0)));
			return res;
		}
	}

};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//invokeLogout()
//@info Used by invoke (@see invoke). Allow to do the appropriate operations
//		when the command sent is LOGOUT. 
//@args CStringList *params The list which defines the command
//@return CString*	The return code which shows if the command has or not suceeded
CString* IradisProtocolServer::invokeLOGOUT(CStringList* params, CTCPDialogThread* tcpThread) {
	cout << "Protocol::invokeLOGOUT" << endl;
	if(params->GetCount() == 1) {
		// Client had been deconnected and  was not logged
		CString * res=new CString();
		server->Notify(new BadCommandEvent("LOGOUT: Client has been disconnected"));
		res->Format(_T("%d"),BASE_KO + CODE_LOGOUT);		
		return res;
	} else {
		CString RemoteLogin = params->GetAt(params->FindIndex(1));
		CUserInfo* user;

		if( RemoteLogin == tcpThread->login && 
			(user = server->userDB->GetUserInfo(RemoteLogin)) != NULL) {
			user->connected = false;
			CString * res = new CString();
			res->Format(_T("%d"),BASE_OK + CODE_LOGOUT);
			server->Notify(new UserOfflineEvent(user->login.GetBuffer(0)));
			cout << "protocol::je vais sender " << *res << endl;
			return res;
		} else {
			CString * res=new CString();
			res->Format(_T("%d"),BASE_KO + CODE_LOGOUT);
			cout << "protocol::je vais sender " << *res << endl;
			server->Notify(new BadCommandEvent("LOGOUT: Bad Logout request."));
			return res;
		}
	}
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//invokeWhois()
//@info Used by invoke (@see invoke). Allow to do the appropriate operations
//		when the command sent is WHOIS.
//@args CStringList *params The list which defines the command
//@return CString*	The return code which shows if the command has or not suceeded
CString* IradisProtocolServer::invokeWHOIS(CStringList* params){
	cout << "Protocol::invokeWHOIS" << endl;
	CString RemoteLogin=params->GetAt(params->FindIndex(1));
	
	CUserInfo* user = server->userDB->GetUserInfo(RemoteLogin);
	
	if (user == NULL){
		CString * res=new CString();
		res->Format(_T("%d"),BASE_KO + CODE_WHOIS);
		cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
		server->Notify(new BadCommandEvent("WHOIS: the user wasn't in the Database"));
		return res;
	}else{
		CString userConnected;
		CString userBanned;

		char szTemp[10];
		_itoa(user->port, szTemp, 10);


		CString ret= "Login : \t\t" + user->login +  "%#Name : \t\t"  + user->user + 
			"%#Lastname : \t" + user->lastname + "%#Status : \t\t" + 
			(user->connected ? "Online" : "Offline") + "%#Banned : \t\t" + 
			(user->banned ? "Yes" : "No") + "%#IP : \t\t" + user->ip + "%#";

		CString tmp="WHOIS "+RemoteLogin+ " completed.";
		server->Notify(new GoodCommandEvent(tmp.GetBuffer(0)));
		return new CString(ret);	
	}

};
/////////////////////////////////////////////////////////////////////////////////////////////	


/////////////////////////////////////////////////////////////////////////////////////////////
//invokeRegister()
//@info Used by invoke (@see invoke). Allow to do the appropriate operations
//		when the command sent is REGISTER.
//@args CStringList *params The list which defines the command
//@return CString*	The return code which shows if the command has or not suceeded
CString* IradisProtocolServer::invokeREGISTER(CStringList* params) {
	cout << "Protocol::invokeREGISTER" << endl;
	CString RemoteLogin=params->GetAt(params->FindIndex(1));
	CString RemotePass=params->GetAt(params->FindIndex(2));
	CString RemoteName=params->GetAt(params->FindIndex(3));
	CString RemoteLastName=params->GetAt(params->FindIndex(4));
	CString RemoteIP=params->GetAt(params->FindIndex(5));
	CString RemotePort=params->GetAt(params->FindIndex(6));

	CUserInfo* user = server->userDB->GetUserInfo(RemoteLogin);

	if (  user != NULL) {
		
		// The User is already registered 
		CString * res = new CString();
		res->Format(_T("%d"),BASE_KO + CODE_REGISTER);
		cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
		server->Notify(new BadCommandEvent("REGISTER: the user "+RemoteLogin+" already exist in the Database"));
		return res;
	} else {
		CUserInfo *ui = new CUserInfo(RemoteLogin, RemotePass, RemoteName,
                     RemoteLastName, false, false, RemoteIP, atoi(RemotePort.GetBuffer(0)));

		server->userDB->RegisterUser(ui);

		CString * res=new CString();
		res->Format(_T("%d"),BASE_OK + CODE_REGISTER);
		cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
		server->Notify(new NewUserEvent(RemoteLogin.GetBuffer(0)));

		return res;
	}

};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//invokeFileinfo()
//@info Used by invoke (@see invoke). Allow to do the appropriate operations
//		when the command sent is FILEINFO.
//@args CStringList *params The list which defines the command
//@return CString*	The return code which shows if the command has or not suceeded
CString* IradisProtocolServer::invokeFILEINFO(CStringList* params){
	cout << "Protocol::invokeREGISTER" << endl;
	CString Filename=params->GetAt(params->FindIndex(1));
	CString Owner=params->GetAt(params->FindIndex(2)); 


	CDocInfo* doc = server->docDB->GetDocInfo(Owner+Filename);


	if (  doc != NULL){
		
		//The document has been found in the database
		CString * res=new CString();
		//res->Format(_T("%d"),BASE_OK + CODE_DOCINFO);4

		res= new CString("Filename :\t" + doc->filename+
			"%#Owner :\t\t"+doc->owner+
			"%#MimeType :\t"+doc->mimetype+
			"%#Description :\t"+doc->description +
			"%#Streamable : \t" + (doc->streamable ? "Yes" : "No"));
		cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
		server->Notify(new GoodCommandEvent("FILEINFO: owner : "+Owner+" File: "+Filename +" infos were found in the Database"));
		return res;

	}else{
		

		CString * res=new CString();
		res->Format(_T("%d"),BASE_KO + CODE_DOCINFO);
		cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
		server->Notify(new BadCommandEvent("FILEINFO : document does not exist"));
		return res;
	}
	


}

/////////////////////////////////////////////////////////////////////////////////////////////
//invokePlaylist()
//@info Used by invoke (@see invoke). Allow to do the appropriate operations
//		when the command sent is FILEINFO.
//@args CStringList *params The list which defines the command
//@return CString*	The return code which shows if the command has or not suceeded
CString* IradisProtocolServer::invokePLAYLIST(CStringList* params){
	cout << "Protocol::invokeREGISTER" << endl;
			
		//The document has been found in the database
		CString * res=new CString();
		res->Format(_T("%d"),BASE_OK + CODE_PLAYLIST);
		
		//Send the playlist in multicast
		server->SendPlaylist();

		cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
		server->Notify(new GoodCommandEvent("PLAYLIST: the playlist was sent in Multicast"));
		return res;

}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//invoke()
//@info This method is used when a command is received from the Client. (TCP mode)
//		Parse the command into a CStringList and then call the appropriate
//		invokation method. (depends of the first argument of the CStringList
//@args CString *request The command received by the Client.
//@return CString*	The return code which shows if the command has or not suceeded
CString* IradisProtocolServer::invoke(CString* request,CTCPDialogThread* tcpThread) {
	
	cout << "Protocol::invoke" << endl;
	
	CStringList *params = parser->extractParams(request);	
	cout << "number:" << params->GetCount() << endl;

	for(int i = 0; i < params->GetCount(); i++){
		POSITION pos=params->FindIndex(i);
		OutputDebugString(("[Param :" + params->GetAt(pos) + "]\n").GetBuffer(0));
		cout << "param: " << (params->GetAt(pos)).GetBuffer(0) << "..." << endl;
	}

	CString commande=params->GetHead();
	

	if (commande == "LOGIN") {
			cout << "Protocol::ON rentre dans login" << endl;
		//We need to ckeck the number of parameters
		if (params->GetCount() == 5) {
			CString * res = invokeLOGIN(params, tcpThread);
			delete params;
			return res;
		}
		else { 
			CString * res=new CString();
			res->Format(_T("%d"),BASE_KO + CODE_BADPARAMETERS);
			cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
			delete params;
			return res;
		}
	}else if (commande == "LOGOUT"){
		//We need to ckeck the number of parameters
		
		if (params->GetCount() >= 1) {
			CString * res = invokeLOGOUT(params, tcpThread);
			delete params;
			return res;
		} else {
		//	cout << "nombre de parametres recus : " << params->GetCount() <<endl;
			CString * res=new CString();
			res->Format(_T("%d"),BASE_KO + CODE_BADPARAMETERS);
			cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
			delete params;
			return res;
		}
	}else if (commande == "WHOIS"){
		//We need to ckeck the number of parameters
		if (params->GetCount() == 2) {
			CString * res = invokeWHOIS(params);
			delete params;
			return res;
		}
		else{ 
			CString * res=new CString();
			res->Format(_T("%d"),BASE_KO + CODE_BADPARAMETERS);
			cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
			delete params;
			return res;
		}
	} else if (commande == "REGISTER"){
		//We need to check the number of parameters
		if (params->GetCount() == 7) {
			CString * res = invokeREGISTER(params);
			if( atoi(*res) == (BASE_OK + CODE_REGISTER) ) {
				params->SetAt( params->FindIndex(0), "LOGIN");
				// remove name
				params->RemoveAt( params->FindIndex(3) );
				// remove lastname
				params->RemoveAt( params->FindIndex(3) );

				for(int i = 0; i < params->GetCount(); i++){
					POSITION pos=params->FindIndex(i);
					OutputDebugString(("[Param RegInvoke:" + params->GetAt(pos) + "]\n").GetBuffer(0));
				}

				invokeLOGIN(params,tcpThread);			
			}
			delete params;
			return res;
		} else {  
			CString * res=new CString();
			res->Format(_T("%d"),BASE_KO + CODE_BADPARAMETERS);
			cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
			delete params;
			return res;
		}
  } else if (commande == "FILEINFO"){
		//We need to check the number of parameters
		if (params->GetCount() == 3) {
			CString * res = invokeFILEINFO(params);
			delete params;
			return res;
    } else { 
			CString * res=new CString();
			res->Format(_T("%d"),BASE_KO + CODE_DOCINFO);
			cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
			delete params;
			return res;
		}
  } else if (commande == "PLAYLIST"){
		//We need to check the number of parameters
		if (params->GetCount() == 1) {
			CString * res = invokePLAYLIST(params);
			delete params;
			return res;
    } else { 
			CString * res=new CString();
			res->Format(_T("%d"),BASE_KO + CODE_PLAYLIST);
			cout << "protocol::je vais sender " << res->GetBuffer(0) << endl;
			delete params;
			return res;
		}
  } else {
		cout << "Protocol::UNKNOWNCOMMAND" << endl;
		CString * tmp=new CString();
		tmp->Format(_T("%d"),BASE_KO + CODE_UNKNOWNCOMMAND);
		cout << "protocol::je vais sender " << tmp->GetBuffer(0) << endl;
		delete params;
		return tmp;
	}

}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////	
//receiveMsgTCP()
//@info Receives a message in TCP mode.
//@args SOCKET socket The socket that is used to communicate with the client (already defined)
//@return CString*	The message received
CString* IradisProtocolServer::receiveMsgTCP(SOCKET socket){
	char *buffer=new char[_SIZE_BUFFER];

	char c; int i=0;
	int size_recv;
	while ((size_recv = recv(socket,&c,1,0)) > 0 && c != '\n' && i < _SIZE_BUFFER) {
		if (c != '\r' ) {
			buffer[i] = c;
			i++;
		}
	}
	if (size_recv == SOCKET_ERROR || i == 0)
		return NULL;

	buffer[i]='\0';

	CString *res = new CString(buffer);
	delete buffer;
	OutputDebugString(("[RECEIVE : " + *res + "]\n").GetBuffer(0));
	return res;
}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//sendMsgTCP()
//@info Semds a message in TCP mode.
//@args SOCKET socket The socket that is used to communicate with the client (already defined)
//@args CString* response The message that will be send to the client.
void IradisProtocolServer::sendMsgTCP(SOCKET socket, CString* response){
	*response += "\n";
	send(socket,*response, response->GetLength(), 0);
	OutputDebugString(("[SEND : " + *response + "]\n").GetBuffer(0));
}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//sendMsgMULTICAST()
//@info Send a message in MULTICAST mode. All the clients who joined the MulticastGroupIP will
//		receive the message (if it isn't lost --> UDP communication <-- )
//@args	CString* buff The message to send
//@args MulticastGroupIP The group of the clients who will receive the message
//@args int remoteport The port where the server will send his MULTICAST msg. 
//		(All the clients must have the same listen port for MULTICAST connection)
//@args SOCKET socket The socket that is used to communicate with the client 
void IradisProtocolServer::sendMsgMULTICAST(CString* buff, char* MulticastGroupIP,int remoteport, SOCKET sock_Listen_MULTICAST) {
	*buff += "\n";
	
	cout << "CMULTICASTRunner::sendMsgMULTICAST() [ENTER]" << endl;
	cout << remoteport << endl;
	//--------------------------------------------------
	//-- Fill in target addr (for a broadcast message)
	SOCKADDR_IN addr_Multicast;

	memset((char *) &addr_Multicast, 0, sizeof(addr_Multicast));
	addr_Multicast.sin_family      = AF_INET;
	addr_Multicast.sin_addr.s_addr = inet_addr(MulticastGroupIP);
	addr_Multicast.sin_port        = htons((short)remoteport);

	if ( sendto(sock_Listen_MULTICAST, buff->GetBuffer(0), buff->GetLength(),0,(sockaddr*)&addr_Multicast,
		 sizeof(addr_Multicast)) == SOCKET_ERROR) {
		cout << "ERROR CMULTICASTRunner::sendMsgMULTICAST() failed.\n";
	} else {
		OutputDebugString("[SEND mcast : "+(*buff)+"]");
		cout << "IradisProtocolServer::sendMsgMULTICAST() OK.\n";
	}


	//--------------------------------------------------
	cout << "IradisProtocolServer::sendMsgMULTICAST() [EXIT]" << endl;
	

};
/////////////////////////////////////////////////////////////////////////////////////////////


