#include "stdafx.h"
#include "Server.h"
#include "ProtocolServer.h"
////////////////////////////////////////////////////////////////////////////////////////////////////////////
////																									////
////	CLASS SERVER RUNNER																				////
////	IRADIS 2001		 																				////
////																									////
////////////////////////////////////////////////////////////////////////////////////////////////////////////


#define MAX_DATAGRAM_SIZE 1000
#define TOKEN '\t'

/////////////////////////////////////////////////////////////////////////////////////////////
//Semaphore
//@info	This semaphore allows to block the send in the Multicast Mode (CMulticastRunner)
//		Every time we want to send in a Multicast mode we just need to release the semaphore.

/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Constructor
//@info	Construct an object of type ServerRunner. Check the version of Winsock
//@args int TCPport    The port where the server will wait for TCP client connections
//		int MULTICASTport	The port which the server will use in order to send Multicast messages
//		int RemoteMULTICASTport	The port where the server will have to send the Multicast message
//		DocInfoManager* dbdoc	A pointer to the document data base 
//		UserInfoManager* dbuser A pointer to the user data base
ServerRunner::ServerRunner(DocInfoManager* dbdoc, UserInfoManager* dbuser){
	cout << "ServerRunner::ServerRunner() [ENTER]" << endl;
	//-----------------------------------------------------
	docDB = dbdoc;
	userDB = dbuser;  
	proto = new IradisProtocolServer(this,new Parser(TOKEN));
	playlist = new CPlayList();
	SendMultiSem = CreateSemaphore(NULL,0,1,"ServerMulticastSemaphore");
	PlaylistMtx = new CMutex(TRUE,NULL,NULL);
	InitSockets();
	fcastSndThread = NULL;
	//-----------------------------------------------------
	cout << "ServerRunner::ServerRunner() [EXIT]" << endl;
};
/////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////
//~ServerRunner()
//@info Destructor
ServerRunner::~ServerRunner(){
	cout << "ServerRunner::~ServerRunner() [ENTER]" << endl;
	//-----------------------------------------------------
	delete proto;
	delete docDB;
	delete userDB;
	//-----------------------------------------------------
	cout << "ServerRunner::~ServerRunner() [EXIT]" << endl;
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Start
//@info Start the server, a Multicast and a TCP Manager thread is created
void ServerRunner::Start(int TCP_port, int MULTICAST_port, CString MULTICAST_ip,int RemoteMULTICAST_port,
						 CString FcastIP, int FcastPort, int FcastRound, int FcastRate) {
	this->FcastIP = FcastIP;
	this->FcastPort = FcastPort;

	InitSockets();
    cout << "ServerRunner::Start() [ENTER]" << endl;
	OutputDebugString("MULTICAST LAUNCH\n");
    //-----------------------------------------------------
    CMULTICASTRunner* pMULTICASTThread = (CMULTICASTRunner*) AfxBeginThread(RUNTIME_CLASS(CMULTICASTRunner),
      THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED);
    pMULTICASTThread->server = this;
    pMULTICASTThread->listenport = MULTICAST_port;
    pMULTICASTThread->remoteport = RemoteMULTICAST_port;
    pMULTICASTThread->ip = MULTICAST_ip;
    Threads.AddTail(pMULTICASTThread);
    pMULTICASTThread->ResumeThread();
    //-----------------------------------------------------
	OutputDebugString("TCP LAUNCH\n");
    //-----------------------------------------------------
    CTCPRunner* pTCPThread = (CTCPRunner*) AfxBeginThread(RUNTIME_CLASS(CTCPRunner),
      THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED);
    pTCPThread->server = this;
    pTCPThread->listenport = TCP_port;
    Threads.AddTail(pTCPThread);		
    pTCPThread->ResumeThread();
    //-----------------------------------------------------
	OutputDebugString("Fcast LAUNCH\n");
    //-----------------------------------------------------
	fcastSndThread = (FcastSenderThread*) AfxBeginThread(RUNTIME_CLASS(FcastSenderThread),
			THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED);
	fcastSndThread->pServer = this;
	fcastSndThread->FcastIP = FcastIP;
	fcastSndThread->FcastPort = FcastPort;
	fcastSndThread->FcastRound = FcastRound;
	fcastSndThread->FcastRate = FcastRate;

	fcastSndThread->ResumeThread();

	OutputDebugString("ServerRunner::Start() [EXIT]\n");
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//SendMULTICAST
//@info Send a multicast message. Notify the send method of CMulticastRunner
//@args CStringList *com	The command to send  
void ServerRunner::SendMULTICAST(CStringList* params){
	cout << "ServerRunner::SendMULTICAST() [ENTER]" << endl;
	//-----------------------------------------------------
	CMULTICASTRunner* pMULTICASTThread = (CMULTICASTRunner *) Threads.GetAt(Threads.FindIndex(0));
	pMULTICASTThread->params = params;
	ReleaseSemaphore(SendMultiSem,1,NULL);
	//-----------------------------------------------------
	cout << "ServerRunner::SendMULTICAST() [Exit]" << endl;
}
/////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////
//SendPlaylist
//@info Send a multicast message. Notify the send method of CMulticastRunner
//@args CStringList *com	The command to send  
void ServerRunner::SendPlaylist(){
	cout << "ServerRunner::SendPlaylist() [ENTER]" << endl;
	//-----------------------------------------------------
	CStringList * playPacket = new CStringList();
	CDocInfo temp;
	int nbCar = 0;
	playPacket->AddTail("PLAYLIST");
	nbCar += CString("PLAYLIST").GetLength()+1; // +1 = delimiter
	
	for (int i = 0; i < playlist->Size(); i++) {
		temp = playlist->GetElementAt(i);
		int nbCarInc = temp.filename.GetLength()+temp.owner.GetLength() + 2;
		if( nbCar+ nbCarInc < MAX_DATAGRAM_SIZE){		
			playPacket->AddTail(temp.filename.GetBuffer(0));
			playPacket->AddTail(temp.owner.GetBuffer(0));
			nbCar += nbCarInc;
		}else{
			break; // size of datagram is reached
		}
	}
	
	SendMULTICAST(playPacket);

	//-----------------------------------------------------
	cout << "ServerRunner::SendPlaylist() [Exit]" << endl;
}
/////////////////////////////////////////////////////////////////////////////////////////////

void ServerRunner::SendFcastInfo(){

	CStringList * FcastInfoPacket = new CStringList();

	FcastInfoPacket->AddTail("FCAST");

	FcastInfoPacket->AddTail(FcastIP);
	
	CString strPort;
	strPort.Format("%d",FcastPort);
	FcastInfoPacket->AddTail(strPort);

	SendMULTICAST(FcastInfoPacket);
}

/////////////////////////////////////////////////////////////////////////////////////////////
//Close
//@info Close the sockets used by the Multicast and the TCP applications, Clean the sockets
void ServerRunner::Close() {
		cout << "ServerRunner::Close() [ENTER]" << endl;
		cout << "kill the Multicast Thread !" << endl; 
		//-----------------------------------------------------
		POSITION pos;
		pos = Threads.GetHeadPosition();
		CMULTICASTRunner* pMULTICASTThread= (CMULTICASTRunner *) Threads.GetAt(pos);
		pMULTICASTThread->Close();
		Threads.RemoveHead();
		//WaitForSingleObject(pMULTICASTThread->m_hThread,INFINITE);
		//-----------------------------------------------------
		OutputDebugString("Multicast Thread removed\n");
		cout << "Multicast Thread removed\n";
		cout << "kill the TCP Thread !" << endl; 
		//-----------------------------------------------------
		pos = Threads.GetHeadPosition();
		CTCPRunner* pTCPThread= (CTCPRunner *) Threads.GetAt(pos);
		pTCPThread->Close();
		Threads.RemoveHead();
		//WaitForSingleObject(pTCPThread->m_hThread,INFINITE);
		cout << "ServerRunner::Close TCP Thread removed\n";
		//-----------------------------------------------------
		OutputDebugString("TCP Thread removed !\n");
		cout << "TCP Thread removed !\n" << endl; 
		//-----------------------------------------------------
		fcastSndThread->StopFcast();
		TerminateThread(fcastSndThread->m_hThread,0);
		OutputDebugString("Fcast Thread removed\n");
		//WSACleanup();

		//-----------------------------------------------------
		cout << "ServerRunner::Close() [EXIT]" << endl;
		
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//ListUsersOnline
//@info Retrieve from the User database all the users who are online
//@return CStringList *	A list of CStrings with the login of the online Users 
CStringList *ServerRunner::ListUsersOnline(){
	cout << "ServerRunner::ListUsersOnline() [ENTER]" << endl;
	CStringList *usersOnline = new CStringList();
	userDB->GetList(usersOnline, true);
	//-----------------------------------------------------
	//-----------------------------------------------------
	cout << "ServerRunner::ListUsersOnline() [EXIT]" << endl;
	return usersOnline;
}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//ListUsersOffline
//@info Retrieve from the User database all the users who are offline
//@return CStringList *	A list of CStrings with the login of the offline Users 
CStringList *ServerRunner::ListUsersOffline(){
	cout << "ServerRunner::ListUsersOffline() [ENTER]" << endl;
	//-----------------------------------------------------
	CStringList *usersOffline = new CStringList();
	userDB->GetList(usersOffline, false);
	//-----------------------------------------------------
	cout << "ServerRunner::ListUsersOffline() [EXIT]" << endl;
	return usersOffline;
}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//getPlayList
//@info 
//@return CStringList *	A list of CStrings with ...
CStringList *ServerRunner::getPlayList() {
	cout << "ServerRunner::getPlayList() [ENTER]" << endl;
	//-----------------------------------------------------

	//-----------------------------------------------------
	cout << "ServerRunner::getPlayList() [EXIT]" << endl;
	return NULL;
}

/////////////////////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////////////////////////////////
////																									////
////	CLASS CTCP RUNNER																				////
////	IRADIS 2001		 																				////
////																									////
////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Win32 Macro
//@info Macro used by WIN32. CTCPRunner is a thread
//		In the .h you must put DECLARE_DYNCREATE(CTCPRunner, CWinThread)
IMPLEMENT_DYNCREATE(CTCPRunner, CWinThread)
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//CTCPRunner()
//@info Empty Constructor
CTCPRunner::CTCPRunner():sock_Listen_TCP(0){
	cout << "CTCPRunner::CTCPRunner() [ENTER]" << endl;
	//-----------------------------------------------------

	//-----------------------------------------------------
	cout << "CTCPRunner::CTCPRunner() [EXIT]" << endl;
	
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//~CTCPRunner()
//@info Destructor
CTCPRunner::~CTCPRunner(){
	cout << "CTCPRunner::~CTCPRunner() [ENTER]" << endl;
	//-----------------------------------------------------
	Close();
	delete server;
	//-----------------------------------------------------
	cout << "CTCPRunner::~CTCPRunner() [EXIT]" << endl;
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//InitTCP()
//@info Create a TCP socket in order to listen for client connections
//		and initialise the client TCP connection.
//		Every time a client connects to the server launch a thread (CTCPThreadDialog) 
//		that will manage his connection. ( ==> Client/Server Multithreaded)
void CTCPRunner::InitTCP(){
	
	cout << "CTCPRunner::InitTCP() [ENTER]" << endl;
	//-----------------------------------------------------

	SOCKADDR_IN		addr_Srv, addr_Cli;
	int				clilen;
	SOCKET			sock_Accept;	
	CString			csTemp;
	
	//-----TCP INITIALISATION----

	//-- Open a TCP socket to listen on 
	if ((sock_Listen_TCP = socket(AF_INET, SOCK_STREAM, 0)) < 0){
		cout << "ERROR: CTCPRunner::InitTCP : Unable to create socket\n";
		exit(0);
	} else {
		cout << "CTCPRunner::InitTCP : TCP listen socket successfully created\n";
	}

	//-- Fill in structure fields for binding to local host
	memset((char *) &addr_Srv, 0, sizeof(addr_Srv));
	addr_Srv.sin_family      = AF_INET;
	addr_Srv.sin_addr.s_addr = htonl(INADDR_ANY);
	addr_Srv.sin_port        = htons((short)listenport);

	//-- Bind it
	if (bind(sock_Listen_TCP, (sockaddr*)&addr_Srv, sizeof(addr_Srv)) < 0){
		cout << "ERROR: CTCPRunner::InitTCP : bind() failed.\n";
    OutputDebugString("ERROR: CTCPRunner::InitTCP : bind() failed.\n");
		Close();
	} else {
		cout << "CTCPRunner::InitTCP : TCP listen socket successfully binded\n";
	}

	//-- Put the socket in listen mode
	if (listen(sock_Listen_TCP, SOMAXCONN) < 0){
		cout << "ERROR: CTCPRunner::InitTCP : listen() failed.\n";
    OutputDebugString("ERROR: CTCPRunner::InitTCP : listen() failed.\n");
		Close();
	} else {
		cout << "CTCPRunner::InitTCP : TCP listen socket successfully binded\n";
	}

	//-- Prepare Client address to receive new data
	memset(&addr_Cli, 0, sizeof(addr_Cli));
	clilen = sizeof(addr_Cli);

	//-----TCP INITIALISATION OVER----

	//-- Wait for an incoming message (Note:  This blocks!)
	while (1) {
		if ((sock_Accept = accept(sock_Listen_TCP, (sockaddr*)&addr_Cli, &clilen)) == INVALID_SOCKET){
			cout << "ERROR: CTCPRunner::InitTCP : Accept() failed.\n";
			OutputDebugString ("ERROR: CTCPRunner::InitTCP : Accept() failed.\n");
			server->Close();
			break;
		} else {
			//-- Hey!  No errors.  We have a new connection
			CTCPDialogThread* pThread = (CTCPDialogThread*) AfxBeginThread(RUNTIME_CLASS(CTCPDialogThread),
        THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED);
			pThread->cliSock = sock_Accept; 
			pThread->addr_Client = addr_Cli;
			pThread->server = server;
			Threads.AddTail(pThread);
			pThread->ResumeThread();
		}
	}
	
	//-----------------------------------------------------
	cout << "CTCPRunner::InitTCP() [EXIT]" << endl;

};
/////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////
//Run
//@info This method is the body of the thread. After InitInstance Run will be called. 
//		Manage the TCP connection ( @see InitTCP() )
int CTCPRunner::Run(){
	cout << "CTCPRunner::Run() [ENTER]" << endl;
	//-----------------------------------------------------
	InitTCP();
	//-----------------------------------------------------
	cout << "CTCPRunner::InitTCP() [EXIT]" << endl;
	return 0;
}
/////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////
//InitInstance
//@info Init the thread. 
//		This method is called after the creation of the thread and before the run method.
//		
BOOL CTCPRunner::InitInstance() {
	cout << "CTCPRunner::InitInstance() [ENTER]" << endl;
	//-----------------------------------------------------
	
	//-----------------------------------------------------
	cout << "CTCPRunner::InitInstance() [EXIT]" << endl;
	
	return TRUE;
	
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//ExitInstance
//@info This method is called when the thread is terminated
//		
int CTCPRunner::ExitInstance(){
	cout << "CTCPRunner::ExitInstance() [ENTER]" << endl;
	//-----------------------------------------------------
	
	//-----------------------------------------------------
	cout << "CTCPRunner::ExitInstance() [EXIT]" << endl;
	return 0;
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////	
//Close
//@info Close the TCP sockets and close all the TCP dialog thread created in order
//		to manage every client connection.
void CTCPRunner::Close(){
	OutputDebugString ("CTCPRunner::Close() [ENTER]\n");	

	//-----------------------------------------------------

	int nbThreads = Threads.GetCount();
	cout << "CTCPRunner::Close Enters\n"<<nbThreads <<"\n";
	for(int i=0; i <nbThreads;i++){
		POSITION pos = Threads.GetHeadPosition();
		CTCPDialogThread* pServerTCPThread=(CTCPDialogThread *) Threads.GetAt(pos);
		pServerTCPThread->Close();
		Threads.RemoveHead();

	}

	shutdown(sock_Listen_TCP,2);
	closesocket(sock_Listen_TCP);
	TerminateThread(this->m_hThread,0);	
	OutputDebugString ("CTCPRunner::Close() [EXIT]\n");		
	//--------------------------------------------
};
/////////////////////////////////////////////////////////////////////////////////////////////	




////////////////////////////////////////////////////////////////////////////////////////////////////////////
////																									////
////	CLASS MULTICAST RUNNER																			////
////	IRADIS 2001		 																				////
////																									////
////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////
//Win32 Macro
//@info Macro used by WIN32. CMULTICASTRunner is a thread
//		In the .h you must put DECLARE_DYNCREATE(CMULTICASTRunner, CWinThread)
IMPLEMENT_DYNCREATE(CMULTICASTRunner, CWinThread)
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//CMULTICASTRunner()
//@info	Empty Constructor
CMULTICASTRunner::CMULTICASTRunner():sock_Listen_MULTICAST(0){
	cout << "CMULTICASTRunner::CMULTICASTRunner() [ENTER]" << endl;
	//------------------------------------------------
	
	//------------------------------------------------
	cout << "CMULTICASTRunner::CMULTICASTRunner() [EXIT]" << endl;
	
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//~CMULTICASTRunner()
//@info	Destructor
CMULTICASTRunner::~CMULTICASTRunner(){
	cout << "CMULTICASTRunner::~CMULTICASTRunner() [ENTER]" << endl;
	//------------------------------------------------
	
	//------------------------------------------------
	cout << "CMULTICASTRunner::~CMULTICASTRunner() [EXIT]" << endl;
	
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Run()
//@info This method is the body of the thread. After InitInstance Run will be called. 
//		Manage the MULTICAST connection. Every time it is requested (semaphore released) 
//		send a packet in MULTICAST mode.
int CMULTICASTRunner::Run(){
	
	CString * command;

	cout << "CMULTICASTRunner::Run() [ENTER]" << endl;
	
	//-----------------------------------------------------
	//-- We can send now
	while(1) {
		WaitForSingleObject(server->SendMultiSem,INFINITE);
		command = server->proto->createCommand(params);
//		delete params;
		server->proto->sendMsgMULTICAST(command,ip.GetBuffer(0),remoteport,sock_Listen_MULTICAST);
		delete command;
	};
	//-----------------------------------------------------
	cout << "CMULTICASTRunner::Run() [EXIT]" << endl;
	return 0;
}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//InitInstance
//@info Init the thread. 
//		This method is called after the creation of the thread and before the run method.
//		(@see InitMULTICAST)
BOOL CMULTICASTRunner::InitInstance(){
	cout << "CMULTICASTRunner::InitInstance() [ENTER]" << endl;
	//-----------------------------------------------------
	InitMULTICAST();
	//-----------------------------------------------------
	cout << "CMULTICASTRunner::InitInstance() [EXIT]" << endl;
	return TRUE;
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//ExitInstance
//@info This method is called when the thread is terminated
int CMULTICASTRunner::ExitInstance(){
	cout << "CMULTICASTRunner::ExitInstance() [ENTER]" << endl;
	//-----------------------------------------------------
	//-----------------------------------------------------
	cout << "CMULTICASTRunner::ExitInstance() [EXIT]" << endl;
	return 0;
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Close
//@info Close the MULTICAST sockets 
void CMULTICASTRunner::Close(){
		cout << "CMULTICASTRunner::Close() [ENTER]" << endl;
		//-----------------------------------------------------
		shutdown(sock_Listen_MULTICAST,2);
		closesocket(sock_Listen_MULTICAST);
		TerminateThread(this->m_hThread,0);		
		//-----------------------------------------------------
		cout << "CMULTICASTRunner::Close() [EXIT]" << endl;
};
/////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////
//InitMULTICAST()
//@info Create a MULTICAST socket in order to hear when a person join a MULTICAST group
void CMULTICASTRunner::InitMULTICAST(){
	
	cout << "CMULTICASTRunner::InitMULTICAST() [ENTER]" << endl;
	//--------------------------------------------------
	
	SOCKADDR_IN		addr_Srv;
	int				val= 1;

	
	//-----UDP MULTICAST INITIALISATION----
	//-- Create UDP Socket
	if ((sock_Listen_MULTICAST = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
		cout << "ERROR CMULTICASTRunner::InitMULTICAST : Unable to create socket\n";
		exit(0);
		Close();
	} else {
		cout << "CMULTICASTRunner::InitMULTICAST : UDP listen socket successfully created\n";
	}

	//-- Fill in structure fields for binding to localhost
	memset((char *) &addr_Srv, 0, sizeof(addr_Srv));
	addr_Srv.sin_family      = AF_INET;
	addr_Srv.sin_addr.s_addr = htonl(INADDR_ANY);
	addr_Srv.sin_port        = htons((short)listenport);

    //-- Bind it
	if (bind(sock_Listen_MULTICAST, (sockaddr*)&addr_Srv, sizeof(addr_Srv)) < 0){
		cout << "ERROR CMULTICASTRunner::InitMULTICAST : bind() failed.\n";
		Close();
		exit(0);		
	} else {
		cout << "CMULTICASTRunner::InitMULTICAST : UDP listen socket successfully binded\n";
	}
	
	//--Allow socket to multicast
	if(setsockopt(sock_Listen_MULTICAST, IPPROTO_IP, IP_MULTICAST_TTL, (char *)&val, sizeof(int))==SOCKET_ERROR){ 
		cout << "ERROR CMULTICASTRunner::InitMULTICAST : setsockopt() failed.\n";
		Close();
		exit(0);
	} else {
		cout << "CMULTICASTRunner::InitMULTICAST : setsockopt OK\n";
	}

	//--------------------------------------------------
	cout << "CMULTICASTRunner::InitMULTICAST() [EXIT]" << endl;
	
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Send
//@info Send a multicast packet in the listeners of a group
//@args char* buff the message sent
//		char* MulticastGroupIP	The address of the group where the Multicast packet will be sent
void CMULTICASTRunner::Send(char* buff, int size, char* MulticastGroupIP) {
	
	cout << "CMULTICASTRunner::Send() [ENTER]" << endl;
	cout << remoteport << endl;
	//--------------------------------------------------
	//-- Fill in target addr (for a broadcast message)
	SOCKADDR_IN addr_Multicast;

	memset((char *) &addr_Multicast, 0, sizeof(addr_Multicast));
	addr_Multicast.sin_family      = AF_INET;
	addr_Multicast.sin_addr.s_addr = inet_addr(MulticastGroupIP);
	addr_Multicast.sin_port        = htons((short)remoteport);

	
	if (sendto(sock_Listen_MULTICAST,buff, size, 0,(sockaddr*)&addr_Multicast, sizeof(addr_Multicast)) == SOCKET_ERROR){
		cout << "ERREUR CMULTICASTRunner::Send : send() failed.\n";
		Close();
		exit(0);
	} else {
		cout << "CMULTICASTRunner::Send : send() OK.\n";
	}

	//--------------------------------------------------
	cout << "CMULTICASTRunner::Send() [EXIT]" << endl;
	

};
/////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////////////
////																									////
////	CLASS TCP DIALOG THREAD																			////
////	IRADIS 2001		 																				////
////																									////
////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Win32 Macro
//@info Macro used by WIN32. CTCPRunner is a thread
//		In the .h you must put DECLARE_DYNCREATE(CTCPDialogThread, CWinThread)
IMPLEMENT_DYNCREATE(CTCPDialogThread, CWinThread)
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////
//CTCPDialogThread()
//@info	Empty Constructor
CTCPDialogThread::CTCPDialogThread():cliSock(0){
	cout << "CTCPDialogThread::CTCPDialogThread() [ENTER]" << endl;
	//--------------------------------------------------

	//--------------------------------------------------
	cout << "CTCPDialogThread::CTCPDialogThread() [EXIT]" << endl;
	
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//~CTCPDialogThread()
//@info	Destructor
CTCPDialogThread::~CTCPDialogThread(){
	cout << "CTCPDialogThread::~CTCPDialogThread() [ENTER]" << endl;
	//--------------------------------------------------

	//--------------------------------------------------
	cout << "CTCPDialogThread::~CTCPDialogThread() [EXIT]" << endl;

};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Run()
//@info This method is the body of the thread. After InitInstance Run will be called. 
//		Manage the CLIENT TCP connection. 
//		Wait for a message from the client. When a message arrives the thread execute
//		the appropriate operations and sends an response to the client
int CTCPDialogThread::Run(){
	
	cout << "CTCPDialogThread::Run() [ENTER]" << endl;
	//--------------------------------------------------
	
	//char *buffer=new char[256];
	//CString * request;
	//-- Client Manager is established 

	while (1) {
				
		CString* command = server->proto->receiveMsgTCP(cliSock);
		if(command == NULL) {
			OutputDebugString("Client connection reset by peer\n");
			command = new CString("LOGOUT" + CString(TOKEN) + login);
			server->proto->invoke(command, this);
			delete command;
			Close();
			return 0;
		}

		cout <<"command received: " << *command << endl;
		CString* toSend = server->proto->invoke(command, this);
		cout << "we send: " << *toSend << endl;
		server->proto->sendMsgTCP(cliSock,toSend);
		delete command;


		if ( toSend->Left(3) == "202" ) {
			//We must close this connection !
			OutputDebugString("Client deconnection\n");
			Close();
			delete toSend;
			return 0;			
		}
		delete toSend;
	}
	
	//--------------------------------------------------
	cout << "CTCPDialogThread::Run() [EXIT]" << endl;
	
	return 0;
}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//InitInstance
//@info Init the thread. 
//		This method is called after the creation of the thread and before the run method.
BOOL CTCPDialogThread::InitInstance() {
	cout << "CTCPDialogThread::InitInstance() [ENTER]" << endl;
	//--------------------------------------------------
	
	cout << "New Thread TCP Manager Launched " << endl;

	//--------------------------------------------------
	cout << "CTCPDialogThread::InitInstance() [EXIT]" << endl;
	
	return TRUE;
}
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//ExitInstance
//@info This method is called when the thread is terminated
int CTCPDialogThread::ExitInstance(){
	cout << "CTCPDialogThread::ExitInstance() [ENTER]" << endl;
	//--------------------------------------------------

	//--------------------------------------------------
	cout << "CTCPDialogThread::ExitInstance() [EXIT]" << endl;
	
	return 0;
};
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//Close
//@info Close the TCP socket used to communicate with the CLIENT
void CTCPDialogThread::Close(){
	cout << "CTCPDialogThread::Close() [ENTER]" << endl;
	//--------------------------------------------------
	
	shutdown(cliSock,2);
	closesocket(cliSock);
	//--------------------------------------------------
	TerminateThread(this->m_hThread,0);
	cout << "CTCPDialogThread::Close() [EXIT]" << endl;
	
};
/////////////////////////////////////////////////////////////////////////////////////////////		

////////////////////////////////////////////////////////////////////////////////////////////////////////////
////																									////
////	MAIN METHOD																						////
////	IRADIS 2001		 																				////
////																									////
////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////		
/*void main(){

	DocInfoManager *dbdoc = new DocInfoManager(".\\doc.db");
	UserInfoManager *dbuser = new UserInfoManager(".\\user.db");
	//cout << "Main : Creating ServerRunner\n";	
	ServerRunner tcps(1550,2000,2001,dbdoc,dbuser);
	
	//cout << "Main : tcps.Start()\n";
	tcps.Start();	
	
	//cout << "Main : Sleeping for 15 s\n";	
	//	Sleep(15000);
	//	cout << "Je ferme le client localhost\n";
	//	tcps.Close("127.0.0.1");
	Sleep(15000);
	CStringList* params = new CStringList();
	params->AddTail("LOGOUT");
	
	cout << "je vais envoyer du MULTICAST" <<endl;
	tcps.SendMULTICAST(params);
	Sleep(INFINITE);
	
	cout << "Main : tcps.Close()\n";



}
*/
/////////////////////////////////////////////////////////////////////////////////////////////		



/////////////////////////////////////////////////////////////////////////////////////////////		
/////////////////////////////////////////////////////////////////////////////////////////////		
//////////////////////////////////EOF////////////////////////////////////////////////////////		
/////////////////////////////////////////////////////////////////////////////////////////////		
/////////////////////////////////////////////////////////////////////////////////////////////		
/////////////////////////////////////////////////////////////////////////////////////////////
